==> mule-runtime-tooling-client-4.2.1-sources.jar: org/mule/tooling/client/internal/MuleArtifactResourcesRegistry.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.tooling.client.internal;

import org.mule.maven.client.api.MavenClient;
import org.mule.runtime.api.meta.MuleVersion;
import org.mule.runtime.container.api.ModuleRepository;
import org.mule.runtime.deployment.model.api.plugin.ArtifactPluginClassLoaderFactory;
import org.mule.runtime.deployment.model.internal.DefaultRegionPluginClassLoadersFactory;
import org.mule.runtime.deployment.model.internal.RegionPluginClassLoadersFactory;
import org.mule.runtime.deployment.model.internal.plugin.BundlePluginDependenciesResolver;
import org.mule.runtime.deployment.model.internal.plugin.PluginDependenciesResolver;
import org.mule.runtime.module.artifact.api.classloader.ArtifactClassLoader;
import org.mule.runtime.module.artifact.api.classloader.DeployableArtifactClassLoaderFactory;
import org.mule.runtime.module.artifact.api.classloader.MuleDeployableArtifactClassLoader;
import org.mule.runtime.module.artifact.api.descriptor.ArtifactDescriptor;
import org.mule.runtime.module.artifact.api.descriptor.ArtifactDescriptorValidatorBuilder;
import org.mule.runtime.module.artifact.api.descriptor.DescriptorLoaderRepository;
import org.mule.runtime.module.deployment.impl.internal.application.ToolingApplicationDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.domain.DomainDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.plugin.ArtifactPluginDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.plugin.ArtifactPluginDescriptorLoader;
import org.mule.tooling.client.internal.application.ApplicationClassLoaderFactory;
import org.mule.tooling.client.internal.application.DomainClassLoaderFactory;

import java.util.List;
import java.util.Optional;

/**
 * Mule resources registry that has implementations for factories and utilities for creating artifacts.
 */
public class MuleArtifactResourcesRegistry {

  private String toolingVersion;
  private Optional<MuleVersion> targetMuleVersion;
  private MavenClient mavenClient;
  private ModuleRepository moduleRepository;

  private ArtifactPluginDescriptorFactory artifactPluginDescriptorFactory;
  private ArtifactPluginDescriptorLoader artifactPluginDescriptorLoader;
  private PluginDependenciesResolver pluginDependenciesResolver;

  private ArtifactClassLoader containerArtifactClassLoader;

  private ToolingApplicationDescriptorFactory applicationDescriptorFactory;
  private DomainDescriptorFactory domainDescriptorFactory;

  private RegionPluginClassLoadersFactory regionPluginClassLoadersFactory;

  private DomainClassLoaderFactory domainClassLoaderFactory;
  private ApplicationClassLoaderFactory applicationClassLoaderFactory;

  /**
   * Creates an instance of the registry.
   */
  public MuleArtifactResourcesRegistry(String toolingVersion, Optional<MuleVersion> targetMuleVersion, MavenClient mavenClient,
                                       ModuleRepository moduleRepository,
                                       ArtifactClassLoader containerArtifactClassLoader) {
    this.toolingVersion = toolingVersion;
    this.targetMuleVersion = targetMuleVersion;
    this.mavenClient = mavenClient;
    this.moduleRepository = moduleRepository;
    this.containerArtifactClassLoader = containerArtifactClassLoader;

    init();
  }

  private void init() {
    // Maven Loaders should use current MavenClient configuration
    DescriptorLoaderRepository descriptorLoaderRepository = new ToolingDescriptorLoaderRepository(mavenClient);

    ArtifactDescriptorValidatorBuilder artifactDescriptorValidatorBuilder = ArtifactDescriptorValidatorBuilder.builder()
        .validateMinMuleVersion().validateMinMuleVersion(() -> toolingVersion)
        .validateMinMuleVersionUsingSemanticVersion();
    artifactPluginDescriptorFactory =
        new ArtifactPluginDescriptorFactory(descriptorLoaderRepository, artifactDescriptorValidatorBuilder);
    artifactPluginDescriptorLoader = new ArtifactPluginDescriptorLoader(artifactPluginDescriptorFactory);

    applicationDescriptorFactory =
        new ToolingApplicationDescriptorFactory(artifactPluginDescriptorLoader, descriptorLoaderRepository,
                                                artifactDescriptorValidatorBuilder);

    domainDescriptorFactory = new DomainDescriptorFactory(artifactPluginDescriptorLoader, descriptorLoaderRepository,
                                                          artifactDescriptorValidatorBuilder);

    regionPluginClassLoadersFactory = new DefaultRegionPluginClassLoadersFactory(new ArtifactPluginClassLoaderFactory(),
                                                                                 moduleRepository);

    pluginDependenciesResolver = new BundlePluginDependenciesResolver(artifactPluginDescriptorFactory);

    domainClassLoaderFactory =
        new DomainClassLoaderFactory(containerArtifactClassLoader, regionPluginClassLoadersFactory, pluginDependenciesResolver);
    applicationClassLoaderFactory =
        new ApplicationClassLoaderFactory(containerArtifactClassLoader, regionPluginClassLoadersFactory,
                                          pluginDependenciesResolver);
  }

  public Optional<MuleVersion> getTargetMuleVersion() {
    return targetMuleVersion;
  }

  public RegionPluginClassLoadersFactory getRegionPluginClassLoadersFactory() {
    return regionPluginClassLoadersFactory;
  }

  public DeployableArtifactClassLoaderFactory<ArtifactDescriptor> newTemporaryArtifactClassLoaderFactory() {
    return new TemporaryArtifactClassLoaderFactory();
  }

  public ArtifactClassLoader getContainerArtifactClassLoader() {
    return containerArtifactClassLoader;
  }

  public ToolingApplicationDescriptorFactory getApplicationDescriptorFactory() {
    return applicationDescriptorFactory;
  }

  public DomainDescriptorFactory getDomainDescriptorFactory() {
    return domainDescriptorFactory;
  }

  public PluginDependenciesResolver getPluginDependenciesResolver() {
    return pluginDependenciesResolver;
  }

  public ArtifactPluginDescriptorLoader getArtifactPluginDescriptorLoader() {
    return artifactPluginDescriptorLoader;
  }

  public DomainClassLoaderFactory getDomainClassLoaderFactory() {
    return domainClassLoaderFactory;
  }

  public ApplicationClassLoaderFactory getApplicationClassLoaderFactory() {
    return applicationClassLoaderFactory;
  }

  /**
   * Creates a class loader instance for a temporary artifact.
   */
  private class TemporaryArtifactClassLoaderFactory
      implements DeployableArtifactClassLoaderFactory<ArtifactDescriptor> {

    /**
     * {@inheritDoc}
     */
    @Override
    public ArtifactClassLoader create(String artifactId, ArtifactClassLoader parent,
                                      ArtifactDescriptor descriptor,
                                      List<ArtifactClassLoader> artifactPluginClassLoaders) {
      return new MuleDeployableArtifactClassLoader(artifactId, descriptor, descriptor.getClassLoaderModel().getUrls(),
                                                   parent.getClassLoader(),
                                                   parent.getClassLoaderLookupPolicy(), artifactPluginClassLoaders);
    }

  }


}

==> mule-runtime-tooling-client-4.2.2-rc2-sources.jar: org/mule/tooling/client/internal/MuleArtifactResourcesRegistry.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.tooling.client.internal;

import org.mule.maven.client.api.MavenClient;
import org.mule.runtime.api.meta.MuleVersion;
import org.mule.runtime.container.api.ModuleRepository;
import org.mule.runtime.deployment.model.api.plugin.ArtifactPluginClassLoaderFactory;
import org.mule.runtime.deployment.model.internal.DefaultRegionPluginClassLoadersFactory;
import org.mule.runtime.deployment.model.internal.RegionPluginClassLoadersFactory;
import org.mule.runtime.deployment.model.internal.plugin.PluginDependenciesResolver;
import org.mule.runtime.module.artifact.api.classloader.ArtifactClassLoader;
import org.mule.runtime.module.artifact.api.classloader.DeployableArtifactClassLoaderFactory;
import org.mule.runtime.module.artifact.api.classloader.MuleDeployableArtifactClassLoader;
import org.mule.runtime.module.artifact.api.descriptor.ArtifactDescriptor;
import org.mule.runtime.module.artifact.api.descriptor.ArtifactDescriptorValidatorBuilder;
import org.mule.runtime.module.artifact.api.descriptor.DescriptorLoaderRepository;
import org.mule.runtime.module.deployment.impl.internal.application.ToolingApplicationDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.domain.DomainDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.plugin.ArtifactPluginDescriptorFactory;
import org.mule.runtime.module.deployment.impl.internal.plugin.ArtifactPluginDescriptorLoader;
import org.mule.runtime.module.deployment.impl.internal.plugin.BundlePluginDependenciesResolver;
import org.mule.tooling.client.internal.application.ApplicationClassLoaderFactory;
import org.mule.tooling.client.internal.application.DomainClassLoaderFactory;

import java.io.File;
import java.util.List;
import java.util.Optional;

/**
 * Mule resources registry that has implementations for factories and utilities for creating artifacts.
 */
public class MuleArtifactResourcesRegistry {

  private String toolingVersion;
  private Optional<MuleVersion> targetMuleVersion;
  private MavenClient mavenClient;
  private ModuleRepository moduleRepository;
  private final File workingDirectory;

  private ArtifactPluginDescriptorFactory artifactPluginDescriptorFactory;
  private ArtifactPluginDescriptorLoader artifactPluginDescriptorLoader;
  private PluginDependenciesResolver pluginDependenciesResolver;

  private ArtifactClassLoader containerArtifactClassLoader;

  private ToolingApplicationDescriptorFactory applicationDescriptorFactory;
  private DomainDescriptorFactory domainDescriptorFactory;

  private RegionPluginClassLoadersFactory regionPluginClassLoadersFactory;

  private DomainClassLoaderFactory domainClassLoaderFactory;
  private ApplicationClassLoaderFactory applicationClassLoaderFactory;

  private DescriptorLoaderRepository descriptorLoaderRepository;

  /**
   * Creates an instance of the registry.
   */
  public MuleArtifactResourcesRegistry(String toolingVersion, Optional<MuleVersion> targetMuleVersion, MavenClient mavenClient,
                                       ModuleRepository moduleRepository,
                                       ArtifactClassLoader containerArtifactClassLoader, File workingDirectory) {
    this.toolingVersion = toolingVersion;
    this.targetMuleVersion = targetMuleVersion;
    this.mavenClient = mavenClient;
    this.moduleRepository = moduleRepository;
    this.containerArtifactClassLoader = containerArtifactClassLoader;
    this.workingDirectory = workingDirectory;

    init();
  }

  public DescriptorLoaderRepository getDescriptorLoaderRepository() {
    return descriptorLoaderRepository;
  }

  private void init() {
    // Maven Loaders should use current MavenClient configuration
    descriptorLoaderRepository = new ToolingDescriptorLoaderRepository(mavenClient);

    ArtifactDescriptorValidatorBuilder artifactDescriptorValidatorBuilder = ArtifactDescriptorValidatorBuilder.builder()
        .validateMinMuleVersion().validateMinMuleVersion(() -> toolingVersion)
        .validateMinMuleVersionUsingSemanticVersion();
    artifactPluginDescriptorFactory =
        new ArtifactPluginDescriptorFactory(descriptorLoaderRepository, artifactDescriptorValidatorBuilder);
    artifactPluginDescriptorLoader = new ArtifactPluginDescriptorLoader(artifactPluginDescriptorFactory);

    applicationDescriptorFactory =
        new ToolingApplicationDescriptorFactory(artifactPluginDescriptorLoader, descriptorLoaderRepository,
                                                artifactDescriptorValidatorBuilder);

    domainDescriptorFactory = new DomainDescriptorFactory(artifactPluginDescriptorLoader, descriptorLoaderRepository,
                                                          artifactDescriptorValidatorBuilder);

    regionPluginClassLoadersFactory = new DefaultRegionPluginClassLoadersFactory(new ArtifactPluginClassLoaderFactory(),
                                                                                 moduleRepository);

    pluginDependenciesResolver = new BundlePluginDependenciesResolver(artifactPluginDescriptorFactory);

    domainClassLoaderFactory =
        new DomainClassLoaderFactory(containerArtifactClassLoader, regionPluginClassLoadersFactory, pluginDependenciesResolver);
    applicationClassLoaderFactory =
        new ApplicationClassLoaderFactory(containerArtifactClassLoader, regionPluginClassLoadersFactory,
                                          pluginDependenciesResolver);
  }

  public Optional<MuleVersion> getTargetMuleVersion() {
    return targetMuleVersion;
  }

  public File getWorkingDirectory() {
    return workingDirectory;
  }

  public RegionPluginClassLoadersFactory getRegionPluginClassLoadersFactory() {
    return regionPluginClassLoadersFactory;
  }

  public DeployableArtifactClassLoaderFactory<ArtifactDescriptor> newTemporaryArtifactClassLoaderFactory() {
    return new TemporaryArtifactClassLoaderFactory();
  }

  public ArtifactClassLoader getContainerArtifactClassLoader() {
    return containerArtifactClassLoader;
  }

  public ToolingApplicationDescriptorFactory getApplicationDescriptorFactory() {
    return applicationDescriptorFactory;
  }

  public DomainDescriptorFactory getDomainDescriptorFactory() {
    return domainDescriptorFactory;
  }

  public PluginDependenciesResolver getPluginDependenciesResolver() {
    return pluginDependenciesResolver;
  }

  public ArtifactPluginDescriptorLoader getArtifactPluginDescriptorLoader() {
    return artifactPluginDescriptorLoader;
  }

  public DomainClassLoaderFactory getDomainClassLoaderFactory() {
    return domainClassLoaderFactory;
  }

  public ApplicationClassLoaderFactory getApplicationClassLoaderFactory() {
    return applicationClassLoaderFactory;
  }

  /**
   * Creates a class loader instance for a temporary artifact.
   */
  private class TemporaryArtifactClassLoaderFactory
      implements DeployableArtifactClassLoaderFactory<ArtifactDescriptor> {

    /**
     * {@inheritDoc}
     */
    @Override
    public ArtifactClassLoader create(String artifactId, ArtifactClassLoader parent,
                                      ArtifactDescriptor descriptor,
                                      List<ArtifactClassLoader> artifactPluginClassLoaders) {
      return new MuleDeployableArtifactClassLoader(artifactId, descriptor, descriptor.getClassLoaderModel().getUrls(),
                                                   parent.getClassLoader(),
                                                   parent.getClassLoaderLookupPolicy(), artifactPluginClassLoaders);
    }

  }


}
